/***************************************************************************
 *   Copyright (C) 2008 by Tamino Dauth                                    *
 *   tamino.dauth@web.de                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
//#include <phonon>

#include "mainwindow.h"
#include "progressdialog.h"
#include "licencedialog.h"

namespace hpp
{

MainWindow::MainWindow() : QMainWindow(), results(0), fileWasSaved(false), fileDialog(0), printDialog(0), translationDirectoryDialog(0), progressDialog(0), licenceDialog(0), languageActionGroup(0)
{
	setupUi(this);

	createLanguageMenuActions();

	qApp->installTranslator(&appTranslator);
	qApp->installTranslator(&qtTranslator);

	connect(saveAsAction, SIGNAL(triggered()), this, SLOT(saveAs()));
	connect(printAction, SIGNAL(triggered()), this, SLOT(print()));
	connect(musicAction, SIGNAL(triggered()), this, SLOT(music()));
	connect(translationAction, SIGNAL(triggered()), this, SLOT(translation()));
	connect(licenceAction, SIGNAL(triggered()), this, SLOT(showLicence()));
	connect(aboutHppAction, SIGNAL(triggered()), this, SLOT(aboutHpp()));
	connect(aboutQtAction, SIGNAL(triggered()), this, SLOT(aboutQt()));
	connect(runPushButton, SIGNAL(pressed()), this, SLOT(run()));
	connect(clearPushButton, SIGNAL(pressed()), this, SLOT(clear()));

	readSettings(); //language will be overwritten if another one was selected

// 	mediaObject = new Phonon::MediaObject(this);
// 	mediaObject->setCurrentSource(Phonon::MediaSource("/sounds/music.wav"));
	//audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
	//Phonon::createPath(mediaObject, audioObject);
// 	mediaObject->play();

/*
	if (QSound::isAvailable())
	{
		backgroundMusic = new QSound((qApp->applicationDirPath() + "/sounds/music.wav"), this);
		backgroundMusic->setLoops(-1);
		backgroundMusic->play();

		connect(musicAction, SIGNAL(triggered()), this, SLOT(music()));
	}
	else
	{
		qDebug("Wave sound system is not available.");
		musicAction->setEnabled(false);
	}
*/
}

MainWindow::~MainWindow()
{
}

void MainWindow::saveAs()
{
	if (results == 0)
	{
		QMessageBox::information(this, tr("Leere Ausgabe"), tr("Die Ausgabe ist leer."));
		return;
	}

	if (fileDialog == 0)
	{
		fileDialog = new QFileDialog(this);
		fileDialog->setAcceptMode(QFileDialog::AcceptSave); 
		fileDialog->setFilter(tr("Textdateien ") + "(*.txt)");
	}

	if (fileDialog->exec() == QDialog::Accepted)
	{
		QFile destination(fileDialog->selectedFiles()[0]);
		
		if (!destination.open(QIODevice::WriteOnly))
			return;
		
		QTextStream out(&destination);
		out << outputTextBrowser->toPlainText(); /// @todo Slow, write line by line!
		fileWasSaved = true;
	}
}

void MainWindow::print()
{
	if (printDialog == 0)
		printDialog = new QPrintDialog(&printer, this);

	if (printDialog->exec() == QDialog::Accepted)
		outputTextBrowser->print(&printer);
}

void MainWindow::music()
{
/*
	if (mediaObject->state() == Phonon::PausedState)
	{
		musicAction->setText(tr("&Musik anhalten"));
		mediaObject->play();
		return;
	}

	musicAction->setText(tr("&Musik abspielen"));
	mediaObject->pause();
*/
}

void MainWindow::translation()
{	
	if (translationDirectoryDialog == 0)
	{
		translationDirectoryDialog = new QFileDialog(this);
		translationDirectoryDialog->setWindowTitle(tr("Übersetzungsverzeichnis wählen"));
		translationDirectoryDialog->setFileMode(QFileDialog::DirectoryOnly);
	}
	
	translationDirectoryDialog->show();
	
	if (translationDirectoryDialog->exec() == QDialog::Accepted)
	{
		QString newPath = translationDirectoryDialog->selectedFiles()[0];
		
		if (!QDir(newPath).exists())
		{
			translation();
			return;
		}
		
		if (newPath == qmPath)
			return;
		
		qmPath = newPath;
			
		if (languageActionGroup != 0)
		{
			delete languageActionGroup;
			languageActionGroup  = 0;
		}
			
		createLanguageMenuActions();
	}
}

void MainWindow::switchLanguage(QAction *action)
{
	QString locale = action->data().toString();
	appTranslator.load("hpp_" + locale, qmPath);
	qtTranslator.load("qt_" + locale, qmPath);

	retranslateUi(this);

	if (licenceDialog != 0)
		licenceDialog->retranslateUi(licenceDialog);

	language = locale;
}

void MainWindow::aboutHpp()
{
	QMessageBox::about(this, tr("Über"), tr(
	"Diese Anwendung wurde<p>"
	"von Tamino Dauth geschrieben.<p>"
	"Sie dient unserem Kaiser Helmut,<p>"
	"genau wie alles andere.<p>Heil dir!<p>"
	));
}

void MainWindow::aboutQt()
{
	QMessageBox::aboutQt(this);
}

void MainWindow::run()
{
	int startValue = startValueSpinBox->value();
	int endValue = endValueSpinBox->value();
	
	if (startValue >= endValue)
	{
		QMessageBox::warning(this, tr("Hinweis"), tr("Der Startwert muss kleiner als der Endwert sein."));
		return;
	}

	if (progressDialog == 0)
		progressDialog = new ProgressDialog(this);

	progressDialog->run(endValue);

	results = 0;

	bool isAPrimeNumber;
	QList<int> primeNumbers;
	
	for (int i = startValue; i <= endValue; ++i)
	{
		progressDialog->changeValue(i);
		isAPrimeNumber = true;
		
		for (QList<int>::const_iterator iterator = primeNumbers.begin(); isAPrimeNumber && iterator != primeNumbers.end(); ++iterator)
		{
			if (progressDialog->canceled())
			{
				qDebug("Canceled");
				return;
			}

			if (0 == i % *iterator)
				isAPrimeNumber = false;
		}
		
		if (isAPrimeNumber)
		{
			primeNumbers.push_back(i);
			++results;
		}
	}

	progressDialog->finish();
	QString outputEntry;

	for (QList<int>::const_iterator iterator = primeNumbers.begin(); iterator != primeNumbers.end(); ++iterator)
		outputEntry += QString("%1<br>").arg(*iterator);

	outputTextBrowser->setHtml(outputEntry);
	resultsLabel->setText(getResultsLabel());
	fileWasSaved = false;
}

void MainWindow::clear()
{
	results = 0;
	resultsLabel->setText(getResultsLabel());
	outputTextBrowser->clear();
}

void MainWindow::showLicence()
{
	if (licenceDialog == 0)
		licenceDialog = new LicenceDialog(this);

	licenceDialog->show();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
	if (!fileWasSaved && results > 0)
	{
		QMessageBox::StandardButton button = QMessageBox::question(this, tr("Ausgabe speichern"), tr("Ausgabe wurde nicht gespeichert."), QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);

		if (button == QMessageBox::Save)
		{
			saveAs();
		}
		else if (button == QMessageBox::Cancel)
		{
			event->ignore();
			return;
		}
	}

	writeSettings();
	QMainWindow::closeEvent(event);
}

void MainWindow::createLanguageMenuActions()
{
	if (languageActionGroup == 0)
		languageActionGroup = new QActionGroup(this);
	
	connect(languageActionGroup, SIGNAL(triggered(QAction*)), this, SLOT(switchLanguage(QAction*)));

	QDir dir(qmPath);
	QStringList fileNames = dir.entryList(QStringList("hpp_*.qm"));

	for (int i = 0; i < fileNames.size(); ++i)
	{
		QString locale = fileNames[i];
		locale.remove(0, (locale.indexOf('_') + 1));
		locale.truncate(locale.lastIndexOf('.'));

		QAction *action = new QAction(QString("&%1 %2").arg(i + 1).arg(locale), languageActionGroup);
		action->setCheckable(true);
		action->setData(locale);
		action->setIcon(QIcon(QString(":/images/translations/%1.png").arg(locale)));

		languageMenu->addAction(action);
		languageActionGroup->addAction(action);
		if (locale == language)
		{
			action->setChecked(true);
			switchLanguage(action);
		}
	}
}

QString MainWindow::getResultsLabel()
{
	switch (results)
	{
		case 0:
			return QString(tr("Keine Ergebnisse:"));
		case 1:
			return QString(tr("1 Ergebnis:"));
		default:
			return QString(tr("%1 Ergebnisse:").arg(results));
	}
	return QString();
}

void MainWindow::writeSettings()
{
	QSettings settings("Tamino Dauth", "hpp");

	settings.setValue("geometry", geometry());
	settings.setValue("startValue", startValueSpinBox->value());
	settings.setValue("endValue", endValueSpinBox->value());
	settings.setValue("language", language);
	settings.setValue("qmPath", qmPath);
}

void MainWindow::readSettings()
{
	QSettings settings("Tamino Dauth", "hpp");

	QRect rect = settings.value("geometry", QRect(QPoint(0, 0), sizeHint())).toRect();
	move(rect.topLeft());
	resize(rect.size());
	startValueSpinBox->setValue(settings.value("startValue", 2).toInt());
	endValueSpinBox->setValue(settings.value("endValue", 3).toInt());
	getResultsLabel();
	language = settings.value("language", "Deutsch").toString();
	qmPath = settings.value("qmPath", qApp->applicationDirPath() + "/translations").toString();
	
	if (!QDir(qmPath).exists())
		translation();
	else
		createLanguageMenuActions();
}

};
